/*
  Doc_Arduino - german Arduino Forum
  Arduino IDE v1.8.13
  Arduino Nano Every
  17.03.2021
  TCBn 16Bit Counter (megaAVR0 TCBn Timer has no OVF Flag)
  License: GNU GPLv3
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <NanoEveryPin.h>
#include <megaAVR0TCBn.h>

InputPin <21> signalEingang; // PD5
TCB <0> timerTCB0;

// --------------------------------------------------------

uint32_t millisNow { millis() };


void setup (void)
{                     
  Serial.begin(250000);             
  Serial.println("\nSTART");
  initPins();
  initEventSystem();
  initTCB0();
}

void loop (void)
{
  zeigeMesswerteTCB(1000);
}

// ****** Funktionen ****************************** //

void zeigeMesswerteTCB (const uint32_t interval)
{
  static uint32_t lastMillis {0};

  if (millis() - lastMillis > interval)
  {
    lastMillis += interval;

    if (timerTCB0.finish)
    {
      uint16_t periode {0};
      uint16_t puls {0};
      ATOMIC_BLOCK (ATOMIC_RESTORESTATE)
      {
        periode = timerTCB0.cnt;
        puls = timerTCB0.ccmp;
      }
      Serial.print("T "); Serial.print(periode); Serial.print(' '); Serial.print('\t');
      Serial.print("P "); Serial.print(puls); Serial.print(' '); Serial.print('\t');
      Serial.print(62.5 * periode / 1000000UL); Serial.print("ms"); Serial.print('\t');
      Serial.print(62.5 * puls / 1000000UL); Serial.print("ms");
      Serial.println();
      timerTCB0.finish = false;
    }
  }
}

void initPins (void)
{ 
  signalEingang.init();
  signalEingang.enablePullup();
}

void initEventSystem (void)
{
  EVSYS.CHANNEL2 = EVSYS_GENERATOR_PORT1_PIN5_gc;   // connect Pin 'PD5' to channel 2
  EVSYS.USERTCB0 = EVSYS_CHANNEL_CHANNEL2_gc;       // connect 'generator 'PD5' with user 'TCB0' via channel 2
}

void initTCB0 (void)
{
  timerTCB0.reset();
  timerTCB0.setCLKDIV1();
  timerTCB0.setModeFRQPW();
  timerTCB0.enableEventInput();
  timerTCB0.enableCAPTInterrupt();
  timerTCB0.enable();
}

ISR(TCB0_INT_vect)
{
  timerTCB0.cnt = TCB0.CNT;         // read out CNT first, then CCMP
  timerTCB0.ccmp = TCB0.CCMP;       
  timerTCB0.finish = true;
  timerTCB0.deleteFlagCAPT();       // clear the interrupt flag
}
