/*
  Doc_Arduino - german Arduino Forum
  Arduino IDE v1.8.13
  C++17 (mindestens C++14)
  Arduino Nano Every
  02.03.2021
  License: GNU GPLv3
  
  Testsuite für Methodentest.
  Es wird über alle 4 TCBn Timer iteriert. Dabei werden alle verfügbaren Methoden getestet.
*/

#include <megaAVR0TCBnArray.h>
#include <utilities.h>

void setup (void)
{
  Serial.begin(250000);
  Serial.println(F("\nStart #### #### #### ####"));

  for (TimerInterface *i : tcb)
  {
    Serial.println();
    showAllReg(i, i-> getRegCTRLA(), i-> getRegCTRLB(), i-> getRegEVCTRL(), i-> getRegINTCTRL(), i-> getRegINTFLAGS(), i-> getRegSTATUS() ); 
    
    Serial.println(F("\nRESET"));
    i-> reset();
    showAllReg(i, i-> getRegCTRLA(), i-> getRegCTRLB(), i-> getRegEVCTRL(), i-> getRegINTCTRL(), i-> getRegINTFLAGS(), i-> getRegSTATUS() ); 
    
    Serial.println(F("\nenable"));          i-> enable();           showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\nisRunning"));       i-> isRunning();        showOneReg(i, i-> getRegSTATUS() ); 
    Serial.println(F("\ndisable"));         i-> disable();          showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\nCLKDIV1"));         i-> setCLKDIV1();       showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\nCLKDIV2"));         i-> setCLKDIV2();       showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\nCLKTCA"));          i-> setCLKTCA();        showOneReg(i, i-> getRegCTRLA() ); 
    Serial.println(F("\nenableSYNCUPD"));   i-> enableSYNCUPD();    showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\ndisableSYNCUPD"));  i-> disableSYNCUPD();   showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\nenableRUNSTDBY"));  i-> enableRUNSTDBY();   showOneReg(i, i-> getRegCTRLA() );  
    Serial.println(F("\ndisableRUNSTDBY")); i-> disableRUNSTDBY();  showOneReg(i, i-> getRegCTRLA() ); 
     
    Serial.println(F("\nINT"));             i-> setModeINT();       showOneReg(i, i-> getRegCTRLB() ); 
    Serial.println(F("\nTIMEOUT"));         i-> setModeTIMEOUT();   showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nCAPT"));            i-> setModeCAPT();      showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nFRQ"));             i-> setModeFRQ();       showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nPW"));              i-> setModePW();        showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nFRQPW"));           i-> setModeFRQPW();     showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nSINGLE"));          i-> setModeSINGLE();    showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nPWM8"));            i-> setModePWM8();      showOneReg(i, i-> getRegCTRLB() );     
    Serial.println(F("\nenableCCMPEN"));    i-> enableCCMPEN();     showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\ndisableCCMPEN"));   i-> disableCCMPEN();    showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nenableCCMPINIT"));  i-> enableCCMPINIT();   showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\ndisableCCMPINIT")); i-> disableCCMPINIT();  showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\nenableASYNC"));     i-> enableASYNC();      showOneReg(i, i-> getRegCTRLB() );  
    Serial.println(F("\ndisableASYNC"));    i-> disableASYNC();     showOneReg(i, i-> getRegCTRLB() );  

    Serial.println(F("\nenableEventInput"));    i-> enableEventInput();   showOneReg(i, i-> getRegEVCTRL() );  
    Serial.println(F("\ndisableEventInput"));   i-> disableEventInput();  showOneReg(i, i-> getRegEVCTRL() );  
    Serial.println(F("\nenableNoiseFilter"));   i-> enableNoiseFilter();  showOneReg(i, i-> getRegEVCTRL() );   
    Serial.println(F("\ndisableNoiseFilter"));  i-> disableNoiseFilter(); showOneReg(i, i-> getRegEVCTRL() );  
    Serial.println(F("\nenableEventEdge"));     i-> enableEventEdge();    showOneReg(i, i-> getRegEVCTRL() );   
    Serial.println(F("\ndisableEventEdge"));    i-> disableEventEdge();   showOneReg(i, i-> getRegEVCTRL() );  
    
    Serial.println(F("\nenableCAPTInt."));      i-> enableCAPTInterrupt();   showOneReg(i, i-> getRegINTCTRL() );  
    Serial.println(F("\ndisableCAPTInt."));     i-> disableCAPTInterrupt();  showOneReg(i, i-> getRegINTCTRL() );  

    Serial.println(F("\ndeleteFlagCAPT"));      i-> deleteFlagCAPT();     showOneReg(i, i-> getRegINTFLAGS() );  
          
    Serial.println();
  }
}

void loop (void)
{
}

// ****** Funktionen ******
void showOneReg(TimerInterface *p, volatile uint8_t *m)
//void showOneReg(auto *p, auto *m)   // benötigt C++14
{  
  switch(p->getAddrBase())
  {
    case 0x0A80: Serial.print(F("TCB0 ")); break; 
    case 0x0A90: Serial.print(F("TCB1 ")); break; 
    case 0x0AA0: Serial.print(F("TCB2 ")); break; 
    case 0x0AB0: Serial.print(F("TCB3 ")); break; 
    default:     Serial.print(F("xxxx ")); break;  
  }
  
  switch(p->getAddrOffset(m))
  {
    case 0x00: Serial.print(F("CTRLA    ")); break; 
    case 0x01: Serial.print(F("CTRLB    ")); break; 
    case 0x04: Serial.print(F("EVCTRL   ")); break; 
    case 0x05: Serial.print(F("INTCTRL  ")); break; 
    case 0x06: Serial.print(F("INTFLAGS ")); break; 
    case 0x07: Serial.print(F("STATUS   ")); break;
    default:   Serial.print(F("xxxxxxxx ")); break;  
  }
  
  formatHEX(Serial, (p->getAddrBase()));     Serial.print('\t');
  formatHEX(Serial, (p->getAddrOffset(m)));  Serial.print('\t');
  formatHEX(Serial, (p->getAddrAbsolut(m))); Serial.print('\t');
  
  formatBIN(Serial, *(m), true);
}

void showAllReg(TimerInterface *p, volatile uint8_t *m1, volatile uint8_t *m2, volatile uint8_t *m3, volatile uint8_t *m4, volatile uint8_t *m5, volatile uint8_t *m6)
//void showAllReg(auto *p, auto *m1, auto *m2, auto *m3, auto *m4, auto *m5, auto *m6)   // benötigt C++14
{
  showOneReg(p, m1); Serial.println();
  showOneReg(p, m2); Serial.println();
  showOneReg(p, m3); Serial.println();
  showOneReg(p, m4); Serial.println();
  showOneReg(p, m5); Serial.println();
  showOneReg(p, m6); Serial.println();
}