/*
  Doc_Arduino - german Arduino Forum
  IDE 1.8.13
  Arduino Nano Every
  02.03.2020
  TCBn 16Bit Counter (megaAVR0 TCBn Timer has no OVF Flag)
  License: GNU GPLv3
*/

#include <avr/interrupt.h>
#include <util/atomic.h>
#include <megaAVR0TCBn.h>

TCB <0> tcb0;

void setup (void)
{                     
  Serial.begin(250000);             
  Serial.println("\nSTART");

  initTimerTBC0();
}

void loop (void)
{
 
}

void initTimerTBC0 (void)
{
  /*
      *** vordefinierte Member zur Verwendung ***
      volatile uint16_t cnt    
      volatile uint16_t ccmp    
      volatile bool finish     

      *** verfügbare Methoden *** 
      reset()                   // setzt alle Timerregister zurück,
                                // ist notwendig für eigene Einstellungen
                                // in der Arduino IDE
                                
      enable()                  // Timer wird gestartet bzw. läuft weiter
      disable()                 // Timer wird gestoppt
      setCLKDIV1()              // Timer läuft mit MCU Takt
      setCLKDIV2()              // Timer läuft mit halben MCU Takt
      setCLKTCA()               // Timer läuft mit Prescaler von Timer 'TCA'
      enableSYNCUPD()           // Timer startet neu wenn TCA neu startet oder überläuft
      disableSYNCUPD()
      enableRUNSTDBY()          // Timer bleibt im Standby Sleep Mode aktiv
      disableRUNSTDBY()
      
      setModeINT()              // Periodic Interrupt Mode
      setModeTIMEOUT()          // Timeout Check Mode
      setModeCAPT()             // Input Capture on Event Mode
      setModeFRQ()              // Input Capture Frequency Measurement Mode
      setModePW()               // Input Capture Pulse Width Measurement Mode
      setModeFRQPW()            // Input Capture Frequency and Pulse Width Measurement Mode
      setModeSINGLE()           // Single Shot Mode (Single Puls Width)
      setModePWM8()             // 8Bit PWM Mode (aus einem 16Bit werden zwei 8Bit Timer)
      enableCCMPEN()            // zugehöriger Timerausgang WOn wird aktiviert
      disableCCMPEN()           // zugehöriger Timerausgang WOn wird deaktiviert (default)
      enableCCMPINIT()          // initialisiert zugehöriger WOn mit '1'
      disableCCMPINIT()         // initialisiert zugehöriger WOn mit '0' (default)
      enableASYNC()             // für Single Shot Mode
      disableASYNC()
  
      enableEventInput()        // aktiviert den Event Eingang
      disableEventInput()
      enableEventEdge()         // konfiguriert auf welcher Flanke der Timer startet und stoppt
      disableEventEdge()        // konfiguriert auf welcher Flanke der Timer startet und stoppt
      enableNoiseFilter()       // aktiviert den Eingangssignalfilter, hat 4 Takte Verzögerung
      disableNoiseFilter()
      
      enableCAPTInterrupt()     // aktiviert den Capture Interrupt
      disableCAPTInterrupt()
      
      deleteFlagCAPT()          // löscht das Capture Interrupt Flag

      bool isRunning()          // gibt den aktuellen Timerstatus zurück
      
      setCounter(uint16_t)      // ändert den aktuellen Timercounter
      setCompare(uint16_t)      // ändert den aktuellen TimerCompare
      
      uint16_t readCounter()    // gibt den aktuellen Timercounter zurück
      uint16_t readCompare()    // gibt den aktuellen TimerCompare zurück
  */
}

// *** ISR Bsp. ***
ISR(TCB0_INT_vect)
{
  tcb0.cnt =  tcb0.readCounter();   // read out CNT first, then CCMP
  tcb0.ccmp = tcb0.readCompare();    
  tcb0.finish = true;
  tcb0.deleteFlagCAPT();            // clear the interrupt flag
}
