/*
  Doc_Arduino - german Arduino Forum
  Programmed and tested with: Arduino IDE 1.8.13 & avr-gcc 10.2.0
  07.03.2021
  License: GNU GPLv3
*/

#pragma once

namespace TCBn
{
  namespace Addr
  {
    struct Offset // Offsets der Registeradressen
    {
      const uint8_t ctrla     = 0x00;
      const uint8_t ctrlb     = 0x01;
      const uint8_t evctrl    = 0x04;
      const uint8_t intctrl   = 0x05;
      const uint8_t intflags  = 0x06;
      const uint8_t status    = 0x07;
      const uint8_t dbgctrl   = 0x08;
      const uint8_t temp      = 0x09;
      const uint8_t cnt       = 0x0A;
      const uint8_t ccmp      = 0x0C;
    }; 
    constexpr Offset addrOffset;


    struct Address // Basis der Registeradressen
    {
      uint16_t  baseAddr;
    };

    #if defined(__AVR_ATmega4809__)
    constexpr Address baseAddr[4] 
    {
    // Base Address  
      {0x0A80},   // TCB0
      {0x0A90},   // TCB1   
      {0x0AA0},   // TCB2
      {0x0AB0},   // TCB3
    };
    #else
      #error "TCBn: Your board has no ATmega4809 controller!".
    #endif
  }
}

/*
  Arduino Nano Every PinOut (ATmega4809)
                  _
               __|_|__    
              |       |  
  (D13)  PE2  +       +  PE1  (D12)  
        3.3V  +       +  PE0  (D11)  
  (AREF) PD7  +       +  PB1  (D10)(~)(TCA0 Channel 1)
  (D14)  PD3  +       +  PB0  (D 9)(~)(TCA0 Channel 0)
  (D15)  PD2  +       +  PE3  (D 8)
  (D16)  PD1  +       +  PA1  (D 7) 
  (D17)  PD0  +       +  PF4  (D 6)(~)(TCB0)
  (D18)  PF2  +       +  PB2  (D 5)(~)(TCA0 Channel 2)
  (D19)  PF3  +       +  PC6  (D 4)  
  (D20)  PD4  +       +  PF5  (D 3)(~)(TCB1)  
  (D21)  PD5  +       +  PA0  (D 2) 
        5.0V  +       +  GND
       RESET  +       +  RESET
         GND  +       +  PC5  (D 0) RX1
         VIN  +       +  PC4  (D 1) TX1      
              |_______|  


  Pin Port  Event-Port  Event-Channel
   0  PC5     0           2 or 3
   1  PC4     0           2 or 3
   2  PA0     0           0 or 1
   3  PF5     1           4 or 5
   4  PC6     0           2 or 3
   5  PB2     1           0 or 1
   6  PF4     1           4 or 5
   7  PA1     0           0 or 1
   8  PE3     0           4 or 5
   9  PB0     1           0 or 1
  10  PB1     1           0 or 1
  11  PE0     0           4 or 5
  12  PE1     0           4 or 5
  13  PE2     0           4 or 5  
  14  PD3     1           2 or 3
  15  PD2     1           2 or 3
  16  PD1     1           2 or 3
  17  PD0     1           2 or 3
  18  PF2     1           4 or 5
  19  PF3     1           4 or 5
  20  PD4     1           2 or 3
  21  PD5     1           2 or 3
*/
