/*
  Doc_Arduino - german Arduino Forum
  Programmed and tested with: Arduino IDE 1.8.13
  02.03.2021
  License: GNU GPLv3
*/

#pragma once

namespace
{
  using Register8  = volatile uint8_t *;
  using Register16 = volatile uint16_t *;
  
  #if defined(__AVR_ATmega4809__)
    #include <include/megaAVR0TCBnRegister.h>
  #else
    #error "Your board has no ATmega4809 controller!".
  #endif
      
  constexpr Register8  regCTRLA    (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.ctrla); }
  constexpr Register8  regCTRLB    (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.ctrlb); }
  constexpr Register8  regEVCTRL   (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.evctrl); }
  constexpr Register8  regINTCTRL  (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.intctrl); }
  constexpr Register8  regINTFLAGS (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.intflags); }
  constexpr Register8  regSTATUS   (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.status); }
  constexpr Register8  regDBGCTRL  (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.dbgctrl); }
  constexpr Register8  regTEMP     (const uint8_t n) { using namespace TCBn::Addr; return (Register8)  (baseAddr[n].baseAddr + addrOffset.temp); }
  constexpr Register16 regCNT      (const uint8_t n) { using namespace TCBn::Addr; return (Register16) (baseAddr[n].baseAddr + addrOffset.cnt); }
  constexpr Register16 regCCMP     (const uint8_t n) { using namespace TCBn::Addr; return (Register16) (baseAddr[n].baseAddr + addrOffset.ccmp); }
  
  constexpr uint8_t CLKSELmask {TCB_CLKSEL1_bm | TCB_CLKSEL0_bm};
  constexpr uint8_t CNTMODEmask {0x07};
}


template<const uint8_t n>   // TCBn number 0...3
class TCB 
{               
  public:
    TCB() = default;
    volatile uint16_t cnt {0};
    volatile uint16_t ccmp {0};
    volatile bool finish {false};  
    
    constexpr void enable()             { *regCTRLA(n) = *regCTRLA(n)  | TCB_ENABLE_bm; }  
    constexpr void disable()            { *regCTRLA(n) = *regCTRLA(n)  & ~TCB_ENABLE_bm; } 
    constexpr void setCLKDIV1()         { *regCTRLA(n) = *regCTRLA(n)  & ~CLKSELmask; }
    constexpr void setCLKDIV2()         { *regCTRLA(n) = (*regCTRLA(n) & ~CLKSELmask) | TCB_CLKSEL_CLKDIV2_gc; }
    constexpr void setCLKTCA()          { *regCTRLA(n) = (*regCTRLA(n) & ~CLKSELmask) | TCB_CLKSEL_CLKTCA_gc; }
    constexpr void enableSYNCUPD()      { *regCTRLA(n) = *regCTRLA(n)  | TCB_SYNCUPD_bm; }
    constexpr void disableSYNCUPD()     { *regCTRLA(n) = *regCTRLA(n)  & ~TCB_SYNCUPD_bm; }
    constexpr void enableRUNSTDBY()     { *regCTRLA(n) = *regCTRLA(n)  | TCB_RUNSTDBY_bm; }
    constexpr void disableRUNSTDBY()    { *regCTRLA(n) = *regCTRLA(n)  & ~TCB_RUNSTDBY_bm; }
    
    constexpr void setModeINT()         { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_INT_gc; }
    constexpr void setModeTIMEOUT()     { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_TIMEOUT_gc; }
    constexpr void setModeCAPT()        { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_CAPT_gc; }
    constexpr void setModeFRQ()         { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_FRQ_gc; }
    constexpr void setModePW()          { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_PW_gc; }
    constexpr void setModeFRQPW()       { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_FRQPW_gc; }
    constexpr void setModeSINGLE()      { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_SINGLE_gc; }
    constexpr void setModePWM8()        { *regCTRLB(n) = (*regCTRLB(n) & ~CNTMODEmask) | TCB_CNTMODE_PWM8_gc; }
    constexpr void enableCCMPEN()       { *regCTRLB(n) = *regCTRLB(n)  | TCB_CCMPEN_bm; }
    constexpr void disableCCMPEN()      { *regCTRLB(n) = *regCTRLB(n)  & ~TCB_CCMPEN_bm; }
    constexpr void enableCCMPINIT()     { *regCTRLB(n) = *regCTRLB(n)  | TCB_CCMPINIT_bm; }
    constexpr void disableCCMPINIT()    { *regCTRLB(n) = *regCTRLB(n)  & ~TCB_CCMPINIT_bm; }
    constexpr void enableASYNC()        { *regCTRLB(n) = *regCTRLB(n)  | TCB_ASYNC_bm; }
    constexpr void disableASYNC()       { *regCTRLB(n) = *regCTRLB(n)  & ~TCB_ASYNC_bm; }

    constexpr void enableEventInput()   { *regEVCTRL(n) = *regEVCTRL(n) | TCB_CAPTEI_bm; }
    constexpr void disableEventInput()  { *regEVCTRL(n) = *regEVCTRL(n) & ~TCB_CAPTEI_bm; }
    constexpr void enableEventEdge()    { *regEVCTRL(n) = *regEVCTRL(n) | TCB_EDGE_bm; }    
    constexpr void disableEventEdge()   { *regEVCTRL(n) = *regEVCTRL(n) & ~TCB_EDGE_bm; }
    constexpr void enableNoiseFilter()  { *regEVCTRL(n) = *regEVCTRL(n) | TCB_FILTER_bm; }
    constexpr void disableNoiseFilter() { *regEVCTRL(n) = *regEVCTRL(n) & ~TCB_FILTER_bm; }
    
    constexpr void enableCAPTInterrupt()  { *regINTCTRL(n) = *regINTCTRL(n) | TCB_CAPT_bm; }  
    constexpr void disableCAPTInterrupt() { *regINTCTRL(n) = *regINTCTRL(n) & ~TCB_CAPT_bm; }  
    
    constexpr void deleteFlagCAPT()     { *regINTFLAGS(n) = *regINTFLAGS(n) | TCB_CAPT_bm; } 
    
    constexpr void reset(void)
    {
      *regCTRLA(n) = 0;
      *regCTRLB(n) = 0;
      *regEVCTRL(n) = 0;
      *regINTCTRL(n) = 0;
      deleteFlagCAPT();
      *regCNT(n) = 0;
      *regCCMP(n) = 0;
    } 
    
    constexpr bool isRunning(void) { return *regSTATUS(n); }
    
    constexpr void setCounter(const uint16_t var) { *regCNT(n) = var;  }
    constexpr void setCompare(const uint16_t var) { *regCCMP(n) = var; }
    
    constexpr uint16_t readCounter(void) { return *regCNT(n); }
    constexpr uint16_t readCompare(void) { return *regCCMP(n); }
    
    // for Debugging and Tests etc.
    constexpr Register8 getRegCTRLA()    { return (Register8) regCTRLA(n); }
    constexpr Register8 getRegCTRLB()    { return (Register8) regCTRLB(n); }
    constexpr Register8 getRegEVCTRL()   { return (Register8) regEVCTRL(n); }
    constexpr Register8 getRegINTCTRL()  { return (Register8) regINTCTRL(n); }
    constexpr Register8 getRegINTFLAGS() { return (Register8) regINTFLAGS(n); }
    constexpr Register8 getRegSTATUS()   { return (Register8) regSTATUS(n); }
    constexpr uint16_t getAddrBase()                 { using namespace TCBn::Addr; return baseAddr[n].baseAddr; }
    constexpr uint8_t  getAddrOffset(Register8 reg)  { return static_cast<uint8_t> (reinterpret_cast<uint16_t>(reg) - getAddrBase() ); }
    constexpr uint16_t getAddrAbsolut(Register8 reg) { return reinterpret_cast<uint16_t>(reg); }
};


