/*
  Doc_Arduino - german Arduino Forum
  Programmed and tested with: Arduino IDE 1.8.13 & avr-gcc 10.2.0
  15.02.2021
  License: GNU GPLv3
*/

#pragma once

#include <megaAVR0TCBn.h>

#define DEBUGMODE   // when commented out "RELEASEMODE", saves a little memory

class TimerInterface // Interface
{
  public:          
    virtual void enable() = 0; 
    virtual void disable() = 0;
    
    virtual void setCLKDIV1() = 0;  
    virtual void setCLKDIV2() = 0;   
    virtual void setCLKTCA() = 0;
    virtual void enableSYNCUPD() = 0;
    virtual void disableSYNCUPD() = 0;
    virtual void enableRUNSTDBY() = 0; 
    virtual void disableRUNSTDBY() = 0;
    
    virtual void setModeINT() = 0;  
    virtual void setModeTIMEOUT() = 0;
    virtual void setModeCAPT() = 0;
    virtual void setModeFRQ() = 0;
    virtual void setModePW() = 0; 
    virtual void setModeFRQPW() = 0;
    virtual void setModeSINGLE() = 0; 
    virtual void setModePWM8() = 0;  
    virtual void enableCCMPEN() = 0; 
    virtual void disableCCMPEN() = 0; 
    virtual void enableCCMPINIT() = 0; 
    virtual void disableCCMPINIT() = 0; 
    virtual void enableASYNC() = 0; 
    virtual void disableASYNC() = 0; 
 
    virtual void enableEventInput() = 0;     
    virtual void disableEventInput() = 0;    
    virtual void enableNoiseFilter() = 0; 
    virtual void disableNoiseFilter() = 0;
    virtual void enableEventEdge() = 0;   
    virtual void disableEventEdge() = 0; 
    
    virtual void enableCAPTInterrupt() = 0;  
    virtual void disableCAPTInterrupt() = 0; 
    virtual void deleteFlagCAPT() = 0; 
    virtual void reset() = 0;
    
    virtual bool isRunning() = 0; 

    virtual void setCounter(const uint16_t var) = 0; 
    virtual void setCompare(const uint16_t var) = 0; 
    
    virtual uint16_t readCounter() = 0; 
    virtual uint16_t readCompare() = 0; 

    #if defined ( DEBUGMODE )
      // for Debugging and Tests etc.
      virtual Register8 getRegCTRLA() = 0;
      virtual Register8 getRegCTRLB() = 0;
      virtual Register8 getRegEVCTRL() = 0;
      virtual Register8 getRegINTCTRL() = 0;
      virtual Register8 getRegINTFLAGS() = 0;
      virtual Register8 getRegSTATUS() = 0;
      virtual uint16_t  getAddrBase() = 0;
      virtual uint8_t   getAddrOffset(Register8 reg) = 0;
      virtual uint16_t  getAddrAbsolut(Register8 reg) = 0; 
    #endif
};
 
template<const uint8_t n>
class Tcb : public TimerInterface
{
  private:
    TCB<n> tcb;

  public:
    virtual void enable()             { tcb.enable(); }  
    virtual void disable()            { tcb.disable(); }  
    
    virtual void setCLKDIV1()         { tcb.setCLKDIV1(); }    
    virtual void setCLKDIV2()         { tcb.setCLKDIV2(); }    
    virtual void setCLKTCA()          { tcb.setCLKTCA(); }  
    virtual void enableSYNCUPD()      { tcb.enableSYNCUPD(); } 
    virtual void disableSYNCUPD()     { tcb.disableSYNCUPD(); } 
    virtual void enableRUNSTDBY()     { tcb.enableRUNSTDBY(); } 
    virtual void disableRUNSTDBY()    { tcb.disableRUNSTDBY(); } 
    
    virtual void setModeINT()         { tcb.setModeINT(); }   
    virtual void setModeTIMEOUT()     { tcb.setModeTIMEOUT(); }  
    virtual void setModeCAPT()        { tcb.setModeCAPT(); }  
    virtual void setModeFRQ()         { tcb.setModeFRQ(); }  
    virtual void setModePW()          { tcb.setModePW(); }  
    virtual void setModeFRQPW()       { tcb.setModeFRQPW(); }  
    virtual void setModeSINGLE()      { tcb.setModeSINGLE(); }  
    virtual void setModePWM8()        { tcb.setModePWM8(); } 
    
    virtual void enableCCMPEN()       { tcb.enableCCMPEN(); }
    virtual void disableCCMPEN()      { tcb.disableCCMPEN(); }
    virtual void enableCCMPINIT()     { tcb.enableCCMPINIT(); }
    virtual void disableCCMPINIT()    { tcb.disableCCMPINIT(); }
    virtual void enableASYNC()        { tcb.enableASYNC(); }
    virtual void disableASYNC()       { tcb.disableASYNC(); }
   
    virtual void enableEventInput()   { tcb.enableEventInput(); }      
    virtual void disableEventInput()  { tcb.disableEventInput(); }    
    virtual void enableNoiseFilter()  { tcb.enableNoiseFilter(); }  
    virtual void disableNoiseFilter() { tcb.disableNoiseFilter(); }  
    virtual void enableEventEdge()    { tcb.enableEventEdge(); }    
    virtual void disableEventEdge()   { tcb.disableEventEdge(); }  
    
    virtual void enableCAPTInterrupt()  { tcb.enableCAPTInterrupt(); }  
    virtual void disableCAPTInterrupt() { tcb.disableCAPTInterrupt(); }  
    virtual void deleteFlagCAPT()       { tcb.deleteFlagCAPT(); }  
    virtual void reset()                { tcb.reset(); }
    
    virtual bool isRunning()            { return tcb.isRunning(); }

    virtual void setCounter(const uint16_t var) { tcb.setCounter(var); }
    virtual void setCompare(const uint16_t var) { tcb.setCompare(var); }
    
    virtual uint16_t readCounter()      { return tcb.readCounter(); }
    virtual uint16_t readCompare()      { return tcb.readCompare(); }

    #if defined ( DEBUGMODE )
      // for Debugging and Tests etc.
      virtual Register8 getRegCTRLA()    { return tcb.getRegCTRLA(); }
      virtual Register8 getRegCTRLB()    { return tcb.getRegCTRLB(); }
      virtual Register8 getRegEVCTRL()   { return tcb.getRegEVCTRL(); }
      virtual Register8 getRegINTCTRL()  { return tcb.getRegINTCTRL(); }
      virtual Register8 getRegINTFLAGS() { return tcb.getRegINTFLAGS(); }
      virtual Register8 getRegSTATUS()   { return tcb.getRegSTATUS(); }
      
      virtual uint16_t getAddrBase()                 { return tcb.getAddrBase(); }
      virtual uint8_t  getAddrOffset(Register8 reg)  { return tcb.getAddrOffset(reg); }
      virtual uint16_t getAddrAbsolut(Register8 reg) { return tcb.getAddrAbsolut(reg); } 
    #endif

};

TimerInterface *tcb[4] =
{
  new Tcb <0>(),   // TCB0
  new Tcb <1>(),   // TCB1
  new Tcb <2>(),   // TCB2
  new Tcb <3>(),   // TCB3
};
    
     

